@@ -38,7 +38,7 @@ __ = getLazyLocalTranslate(_)
 
 from calculate.lib.utils.files import (process, readFile, processProgress,
                                        makeDirectory, getProgPath)
-from calculate.lib.datavars import DataVarsError, CriticalError
+from calculate.lib.datavars import DataVarsError, CriticalError, Variable
 from calculate.lib.utils.content import getCfgFiles
 from itertools import *
 
@@ -86,6 +86,8 @@ class CommonMethods(MethodsInterface):
             newdata = readFile(data[i_cfgname])
             pattern = "%s/._cfg????_%s" % (os.path.dirname(orig),
                                            os.path.basename(orig))
+            answ_map = {'usenew': 'use new', 'skip': 'next'}
+            dispatch_var = self.clVars.Get('cl_dispatch_conf')
             for fn in glob.glob(pattern):
                 try:
                     if fn == data[i_cfgname]:
@@ -96,6 +98,8 @@ class CommonMethods(MethodsInterface):
             if (self.clVars.Get('cl_autoupdate_set') == 'on' or
                         origdata == newdata):
                 answ = "use new"
+            elif dispatch_var in answ_map:
+                answ = answ_map.get(dispatch_var)
             else:
                 orig_content = normalize_config(readFile(orig))
                 new_content = normalize_config(readFile(data[i_cfgname]))
@@ -162,7 +166,7 @@ class CommonMethods(MethodsInterface):
         return True
 
     def applyTemplates(self, target=None, useClt=None, cltFilter=False,
-                       root=None, useDispatch=True):
+                       root=None, useDispatch=True, critical=False):
         """
         Применить шаблоны.
         
@@ -199,7 +203,8 @@ class CommonMethods(MethodsInterface):
                                     printWARNING=self.printWARNING,
                                     askConfirm=self.askConfirm,
                                     dispatchConf=dispatch,
-                                    printERROR=self.printERROR)
+                                    printERROR=self.printERROR,
+                                    critical=critical)
         try:
             cl_templ.applyTemplates()
             if cl_templ.hasError():
@@ -219,15 +224,18 @@ class CommonLink(object):
     """
     com = None
 
+    @staticmethod
+    def link_object(source, target):
+        for fn in (x for x in dir(CoreWsdl.Common) if not x.startswith("_")):
+            if hasattr(source, fn):
+                setattr(target, fn, getattr(source, fn))
+
     def set_link(self, com):
         """
         Установить связь с Common объектом
         """
         self.com = com
-        # подключить все методы Common
-        for fn in (x for x in dir(CoreWsdl.Common) if not x.startswith("_")):
-            if hasattr(com, fn):
-                setattr(self, fn, getattr(com, fn))
+        self.link_object(com, self)
 
 
 class ActionError(Exception):
@@ -459,7 +467,7 @@ class Action(MethodsInterface):
             _type = dv.getInfo(param).type
             if _type == "int":
                 return dv.GetInteger(param)
-            if _type == "bool":
+            if _type in ("bool", "boolauto"):
                 return dv.GetBool(param)
             return dv.Get(param)
 
@@ -493,7 +501,7 @@ class Action(MethodsInterface):
             else:
                 val = dv.Get(var)
             if type(val) in (list, tuple):
-                val = ",".join(val)
+                val = ", ".join(val)
             if func:
                 if hasattr(tt, func):
                     val = getattr(tt, func)(val)
@@ -540,9 +548,9 @@ class Action(MethodsInterface):
         Герератор задач (поддержка линейной обработки задач в группах)
         """
         for task in self.tasks:
-            if "group" in task:
+            if "group" in task or "tasks" in task:
                 if all(self.get_condition_context(task, result).values()):
-                    self.group_name = task["group"]
+                    self.group_name = task.get("group", "")
                     for action in task["tasks"]:
                         yield action
                     if not self.group_name:
@@ -928,12 +936,21 @@ class CoreWsdl(CoreServiceInterface):
         from loaded_methods import LoadedMethods
 
         results = []
+        dv = DataVars()
+        dv.importVariables()
+
         for item in LoadedMethods.guiMethods:
-            for i in range(0, len(LoadedMethods.guiMethods[item]), 3):
-                temp = [item]
-                for j in range(3):
-                    temp.append(LoadedMethods.guiMethods[item][i + j])
-                results.append(temp)
+            for i in range(0, len(LoadedMethods.guiMethods[item]), 4):
+                if LoadedMethods.guiMethods[item][i + 3]:
+                    method_on = LoadedMethods.guiMethods[item][i + 3](dv.Get)
+                else:
+                    method_on = True
+                if method_on:
+                    temp = [item]
+                    for j in range(3):
+                        temp.append(LoadedMethods.guiMethods[item][i + j])
+                    results.append(temp)
+        dv.close()
         return results
 
     # get available sessions
@@ -1072,7 +1089,8 @@ class CoreWsdl(CoreServiceInterface):
             self.addMessage(type='progress', id=id)
 
         def printTable(self, table_name, head, body, fields=None,
-                       onClick=None, addAction=None):
+                       onClick=None, addAction=None, step=None,
+                       records=None):
             id = random.randint(1, self.Num)
             while id in self.table_dict:
                 id = random.randint(1, self.Num)
@@ -1081,8 +1099,8 @@ class CoreWsdl(CoreServiceInterface):
 
             table = Table(head=head, body=map(lambda x: map(str, x), body),
                           fields=fields,
-                          onClick=onClick, addAction=addAction,
-                          values=None)
+                          onClick=onClick, addAction=addAction, step=step,
+                          values=None, records=records)
             self.table_dict[id] = table
             self.addMessage(type='table', message=table_name, id=id)
 
@@ -1105,23 +1123,7 @@ class CoreWsdl(CoreServiceInterface):
                     raise
                 print _(("%s:" % type) + str(message))
 
-        def printSUCCESS(self, message='', onlyShow=None):
-            self.addMessage(type='normal', message=message,
-                            onlyShow=onlyShow)
-
-        def printPre(self, message='', onlyShow=None):
-            self.addMessage(type='pre', message=message,
-                            onlyShow=onlyShow)
-
-        def printDefault(self, message='', onlyShow=None):
-            self.addMessage(type='plain', message=message,
-                            onlyShow=onlyShow)
-
-        def printWARNING(self, message, onlyShow=None):
-            self.addMessage(type='warning', message=message,
-                            onlyShow=onlyShow)
-
-        def printERROR(self, message='', onlyShow=None):
+        def dropProgress(self):
             perc = self.getProgress()
             if perc == 0:
                 self.setProgress(100)
@@ -1130,6 +1132,29 @@ class CoreWsdl(CoreServiceInterface):
             else:
                 # self.setProgress(-100)
                 self.setProgress(perc)
+
+        def printSUCCESS(self, message='', onlyShow=None):
+            self.dropProgress()
+            self.addMessage(type='normal', message=message,
+                            onlyShow=onlyShow)
+
+        def printPre(self, message='', onlyShow=None):
+            self.dropProgress()
+            self.addMessage(type='pre', message=message,
+                            onlyShow=onlyShow)
+
+        def printDefault(self, message='', onlyShow=None):
+            self.dropProgress()
+            self.addMessage(type='plain', message=message,
+                            onlyShow=onlyShow)
+
+        def printWARNING(self, message, onlyShow=None):
+            self.dropProgress()
+            self.addMessage(type='warning', message=message,
+                            onlyShow=onlyShow)
+
+        def printERROR(self, message='', onlyShow=None):
+            self.dropProgress()
             self.addMessage(type='error', message=message,
                             onlyShow=onlyShow)
 
@@ -1431,8 +1456,9 @@ class CoreWsdl(CoreServiceInterface):
 
 def create_symlink(data_path, old_data_path):
     meths = LoadedMethods.conMethods
-    path_to_link = '/usr/sbin'
-    path_to_user_link = '/usr/bin'
+    path_to_link = '/usr/bin'
+    core_wrapper = "/usr/libexec/calculate/cl-core-wrapper"
+    #path_to_user_link = '/usr/bin'
     old_symlinks_file = os.path.join(old_data_path, 'conf/symlinks')
     symlinks_file = os.path.join(data_path, 'conf/symlinks')
     if not os.path.exists(os.path.join(data_path, 'conf')):
@@ -1446,10 +1472,7 @@ def create_symlink(data_path, old_data_path):
         os.unlink(old_symlinks_file)
     fd = open(symlinks_file, 'a')
     for link in meths:
-        if meths[link][1]:
-            link_path = os.path.join(path_to_user_link, link)
-        else:
-            link_path = os.path.join(path_to_link, link)
+        link_path = os.path.join(path_to_link, link)
         if os.path.islink(link_path):
             continue
         if os.path.isfile(link_path):
@@ -1457,7 +1480,11 @@ def create_symlink(data_path, old_data_path):
             print red + link_path + _(' is a file, not a link!')
             continue
         try:
-            os.symlink(os.path.join(path_to_link, 'cl-core'), link_path)
+            if (os.path.islink(link_path) and
+                        os.readlink(link_path) != core_wrapper):
+                os.unlink(link_path)
+                print "REFRESH OLD", link_path
+            os.symlink(core_wrapper, link_path)
             fd.write(link_path + '\n')
         except OSError, e:
             print e.message
@@ -1469,10 +1496,7 @@ def create_symlink(data_path, old_data_path):
     temp_text_file = ''
     for line in readLinesFile(symlinks_file):
         cmdname = os.path.basename(line)
-        if (cmdname not in meths.keys() or
-                    line.startswith(path_to_link) and meths[cmdname][1] or
-                    line.startswith(path_to_user_link) and
-                    not meths[cmdname][1]):
+        if cmdname not in meths.keys() or not line.startswith(path_to_link):
             if os.path.islink(line):
                 os.unlink(line)
                 print _('Symlink %s deleted') % line
@@ -1582,7 +1606,8 @@ class WsdlMeta(type):
         for gr in kwargs['groups']:
             gr(group)
         # if "brief" in kwargs:
-        if "cl_page_count" not in d:
+        #if "cl_page_count" not in d:
+        if True:
             d["CheckOnly"] = Boolean
         return d
 
@@ -1600,7 +1625,9 @@ class WsdlMeta(type):
             return self.callAction(sid, info, logicClass=kwargs['logic'],
                                    actionClass=kwargs['action'],
                                    method_name=kwargs['method_name'],
-                                   callbackRefresh=callback_refresh)
+                                   callbackRefresh=callback_refresh,
+                                   invalidators=kwargs.get('invalidators', None)
+                                   )
 
         wrapper.func_name = kwargs['method_name']
         func = LoadedMethods.core_method(category=kwargs.get('category', None),
@@ -1609,7 +1636,9 @@ class WsdlMeta(type):
                                          gui=kwargs['gui'],
                                          user=kwargs.get('user', False),
                                          command=kwargs.get('command', None),
-                                         rights=kwargs['rights'])(wrapper)
+                                         rights=kwargs['rights'],
+                                         depends=kwargs.get('depends', ()))(
+            wrapper)
         if "--start" in sys.argv:
             info_obj = mcs.create_info_obj(**kwargs)
             info_class = type("%sInfo" % kwargs["method_name"], (CommonInfo,),
@@ -1629,6 +1658,9 @@ class WsdlMeta(type):
             # если значение функция
             if callable(v):
                 v = v(dv)
+            else:
+                if isinstance(v, (str, unicode)):
+                    v = Variable._value_formatter.format(v, dv.Get)
             dv.Set(k.strip('!'), v, force=k.endswith('!'))
 
     @classmethod
@@ -1650,7 +1682,8 @@ class WsdlMeta(type):
                 for gr in kwargs['groups']:
                     gr(group)
             if not dv:
-                dv = getattr(self, "%s_vars" % kwargs["method_name"])()
+                dv = getattr(self, "%s_vars" % kwargs["method_name"])(
+                    params=params)
                 if hasattr(params, "clienttype"):
                     if params.clienttype == 'gui' and "guivars" in kwargs:
                         mcs.modify_datavars(dv, kwargs['guivars'])
@@ -1664,6 +1697,7 @@ class WsdlMeta(type):
                     lang = dv.Get('install.os_install_locale_lang')
                     self.set_cache(sid, "install", "lang", lang, smart=False)
                 dv.processRefresh()
+
             self.set_cache(sid, kwargs["method_name"], "vars", dv, smart=False)
             if "brief" in kwargs and "name" in kwargs['brief']:
                 brief_label = str(kwargs['brief']['name'])
@@ -1688,7 +1722,7 @@ class WsdlMeta(type):
         Конструктор для создания метода описания параметров
         """
 
-        def wrapper(self, dv=None):
+        def wrapper(self, dv=None, params=None):
             if not dv:
                 dv = DataVars()
                 dv.importVariables()
@@ -1696,6 +1730,10 @@ class WsdlMeta(type):
                     'calculate.%s.variables' % kwargs['datavars'])
                 dv.defaultModule = kwargs['datavars']
                 dv.flIniFile()
+                if params and params.help_set:
+                    dv.Set('cl_help_set', "on", force=True)
+                if params and params.dispatch_usenew:
+                    dv.Set('cl_dispatch_conf', "usenew", force=True)
                 mcs.modify_datavars(dv, kwargs['setvars'])
             # созданием группы переменных из datavars согласно параметрам groups
             for groupfunc in kwargs['groups']:
@@ -1725,3 +1763,43 @@ def clearDataVars(func):
             WsdlMeta.close_datavars()
 
     return wrapper
+
+class CustomButton(object):
+    @classmethod
+    def sort_argv(cls, args):
+        behavior = []
+        condition = None
+        for arg in args:
+            if callable(arg):
+                condition = arg
+            else:
+                behavior.append(arg)
+        return (behavior or None), condition
+
+    @classmethod
+    def run_method(cls, method_name, id, label, *args):
+        behavior, condition = cls.sort_argv(args)
+        return id, label, method_name, "button", behavior, condition
+
+    @classmethod
+    def open_method(cls, method_name, id, label, *args):
+        behavior, condition = cls.sort_argv(args)
+        return id, label, method_name, "button_view", behavior, condition
+
+    @classmethod
+    def next_button(cls, id=None, label=None):
+        return id, label, None, "button_next"
+
+    class Behavior(object):
+        @classmethod
+        def link(cls, source=None, target=None):
+            return "%s=%s" % (target, source)
+
+        @classmethod
+        def linkerror(cls, source=None, target=None):
+            return "%s->%s" % (source, target)
+
+        @classmethod
+        def setvalue(cls, variable=None, value=None):
+            return "%s!=%s" % (variable, value)
+
